/* Copyright (C) 2005-2019 Shugo Maeda <shugo@ruby-lang.org> and Charlie Savage <cfis@savagexi.com>
   Please see the LICENSE file for copyright and distribution information */

#ifndef __BF_MEASUREMENT_H__
#define __BF_MEASUREMENT_H__

#include "blackfire.h"

extern VALUE mMeasure;

/* Helper structure to return 4 doubles */
struct four_doubles_t
{
    double wt;
    double pr;
    double al;
    double mm;
};

typedef struct four_doubles_t (*get_measurement)(void *measurer_ptr, rb_trace_arg_t *trace_arg);

typedef struct prof_measurer_t
{
    bool flag_cpu;
    get_measurement measure;
    struct four_doubles_t multipliers;
} prof_measurer_t;

/* Callers and callee information for a method. */
typedef struct prof_measurement_t
{
    double wt_total_time;
    double wt_self_time;
    double wt_wait_time;

    double pr_total_time;
    double pr_self_time;
    double pr_wait_time;

    double al_total_time;
    double al_self_time;
    double al_wait_time;

    double mm_self_time;
    double mm_wait_time;

    int called;
    VALUE object;
} prof_measurement_t;

prof_measurer_t *prof_get_measurer(void);
struct four_doubles_t prof_measure(prof_measurer_t *measurer, rb_trace_arg_t *trace_arg);

prof_measurement_t *prof_measurement_create(void);
void prof_measurement_free(prof_measurement_t *measurement);
VALUE prof_measurement_wrap(prof_measurement_t *measurement);
prof_measurement_t *prof_get_measurement(VALUE self);
void prof_measurement_mark(void *data);

void rp_init_measure(void);

#endif //__BF_MEASUREMENT_H__
