module Blackfire

  class BlackfireTracesPrinter < AbstractPrinter

    MULTIPLIER_FOR_TIME = 1_000_000
    MULTIPLIER_FOR_MEMORY = 1

    def generate_key(caller, callee)
      if !callee.nil?
        result = "#{caller}==>#{callee}"
      else
        result = caller.to_s
      end

      result
    end

    def generate_trace(key, call_tree)
      format(
          "%s//%d %s %s\n",
          key,
          call_tree.called,
          (call_tree.wall_time_total_time * MULTIPLIER_FOR_TIME).to_i.to_s,
          (call_tree.process_total_time * MULTIPLIER_FOR_TIME).to_i.to_s,
      )
    end

    def print_methods(thread)
      root = thread.call_tree
      metrics = "Graph-Root-Id: #{root.target.key}\n"

      # add root
      key = generate_key(root.target.key, nil)
      metrics << generate_trace(key, root)

      # add children
      thread.methods.each do |method|
          method.call_trees.callees.each do |child|
          key = generate_key(method.key, child.target.key)
          metrics << generate_trace(key, child)
          end
      end
      @output << metrics
    end
  end
end
